﻿using Kivii.Linq;
using Kivii.Samples.Entities;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Samples.Transforms
{
    #region CRUDQ
    //[RequiresAnyRole(SystemRoles.Everyone)]
    public class SampleQuery : RestfulQuery<Sample>
    {
        public List<Guid> BizKvids { get; set; }

        public bool IncludeLocation { get; set; }

        public override bool OnPreRestfulQuery(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulQueryResponse<Sample> rtns, ref Expression<Func<Sample, bool>> where)
        {
            if (!BizKvids.IsNullOrEmpty()) where = o => Sql.In(o.BizKvid, BizKvids);
            return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
        }
        public override bool OnPostRestfulQuery(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulQueryResponse<Sample> rtns)
        {
            if (IncludeLocation)
            {
                foreach (var item in rtns.Results)
                {
                    item.CurrentLocation = item.GetCurrentLocation();
                }
            }
            return base.OnPostRestfulQuery(req, res, dbConnection, rtns);
        }
    }
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SampleCreate : RestfulCreate<Sample>
    {
        public override bool OnPreRestfulCreate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulCreateResponse<Sample> rtns)
        {
            if (Item.Type != SampleType.Package) throw new Exception("只能新建包裹类型样品!");
            Item.Category = SampleType.Package.ToString();
            return base.OnPreRestfulCreate(req, res, dbConnection, rtns);
        }
    }
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SampleUpdate : RestfulUpdate<Sample>
    {
    }
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SampleRead : RestfulRead<Sample>
    {
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    public class SampleDelete : RestfulDelete<Sample>
    {
    }
    #endregion

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description ="生成样品")]
    public class SampleAccept : RestfulExecution<Sample>
    {
        public Sample Item { get; set; }//要创建的样品信息

        public Location Location { get; set; }//地点位置信息,只需要InternalCode

        public Route Route { get; set; }//自定义路由信息,可不填写

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Item == null).ThrowIfTrue("缺少要创建的样品实例!");
            (Item.Name.IsNullOrEmpty()).ThrowIfTrue("缺少样品名称信息!");
            (Location == null).ThrowIfTrue("缺少目的地点配置信息!");
            (Location.InternalCode.IsNullOrEmpty()).ThrowIfTrue("缺少目的地点配置InternalCode信息");
            var conn = KiviiContext.GetOpenedDbConnection<Location>();
            var local= conn.Single<Location>(o => o.InternalCode == Location.InternalCode);
            (local == null).ThrowIfTrue($"未找到{Location.InternalCode}目的地点配置信息!");

            var rtns = new RestfulCreateResponse<Sample>();
            rtns.Results = new List<Sample>();

            var accept = Item.Accept(local, Route);
            rtns.Results.Add(accept);
            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "样品拆分")]
    public class SampleSplit:RestfulExecution<Sample>
    {
        public Guid Kvid { get; set; }//要拆分的样品Kvid

        public List<Sample> Samples { get; set; }//拆分出的子样品信息

        public Location Location { get; set; }//地点位置信息,只需要InternalCode

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Kvid == Guid.Empty).ThrowIfTrue("缺少要拆分的样品Kvid");
            Samples.ThrowIfNullOrEmpty("缺少拆分的样品信息");
            (Location == null).ThrowIfTrue("缺少目的地点配置信息!");
            (Location.InternalCode.IsNullOrEmpty()).ThrowIfTrue("缺少目的地点配置InternalCode信息");

            var conn = KiviiContext.GetOpenedDbConnection<Location>();
            var local = conn.Single<Location>(o => o.InternalCode == Location.InternalCode);
            (local == null).ThrowIfTrue($"未找到{Location.InternalCode}目的地点配置信息!");

            var sample = new Sample();
            sample.Kvid = Kvid;

            var splited = sample.Split(Samples, local);
            var rtns = new RestfulCreateResponse<Sample>();
            rtns.Results = new List<Sample>();
            rtns.Results.AddRange(splited);

            return rtns;
        }
    }

    //[RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "样品流转")]
    public class SampleAssign : RestfulExecution<Sample>
    {
        public Guid Kvid { get; set; }//要流转样品信息
        public List<Guid> Kvids { get; set; }//要流转样品信息

        public Location Location { get; set; }//地点位置信息,只需要InternalCode

        public Route Route { get; set; }//自定义路由信息,可不填写

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Kvid == Guid.Empty && Kvids.IsNullOrEmpty()).ThrowIfTrue("缺少要流转的样品信息!");
            Location.ThrowIfNull("缺少目的地点信息!");
            (Location.InternalCode.IsNullOrEmpty()).ThrowIfTrue("缺少目的地点配置InternalCode信息");

            var conn = KiviiContext.GetOpenedDbConnection<Location>();
            var local = conn.Single<Location>(o => o.InternalCode == Location.InternalCode);
            (local == null).ThrowIfTrue($"未找到{Location.InternalCode}目的地点配置信息!");

            if (Kvids.IsNullOrEmpty()) Kvids = new List<Guid>();
            if (Kvid != Guid.Empty) Kvids.Add(Kvid);

            var rtns = new RestfulUpdateResponse<Sample>();
            rtns.Results = new List<Sample>();

            foreach (var kvid in Kvids)
            {
                var sample = new Sample();
                sample.Kvid = kvid;

                var assign = sample.Assign(local, Route);
                rtns.Results.Add(assign);
            }
            return rtns;
        }
    }

    //[RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "所在地样品")]
    public class SampleLocationQuery : RestfulQuery<Sample>
    {
        public Guid LocationKvid { get; set; }
        public string LocationInternalCode { get; set; }

        private Location location;

        public override bool OnPreRestfulQuery(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulQueryResponse<Sample> rtns, ref Expression<Func<Sample, bool>> where)
        {
            (LocationKvid == Guid.Empty && LocationInternalCode.IsNullOrEmpty()).ThrowIfTrue("请传入要查询的地点信息!");
            //Location location = null;
            if (LocationKvid != Guid.Empty) location = dbConnection.SingleById<Location>(LocationKvid);
            if (location == null)
            {
                if (!LocationInternalCode.IsNullOrEmpty()) location = dbConnection.Single<Location>(o => o.InternalCode == LocationInternalCode);
                location.ThrowIfNull("未找到指定的地点信息!");
            }
            var queryRoutes = dbConnection.From<Route>();
            queryRoutes.Where(o => o.CurrentLocationKvid == location.Kvid && o.NextLocationKvid == Guid.Empty);
            queryRoutes.Select(o => o.SampleKvid);
            where = o => Sql.In(o.Kvid, queryRoutes);
            return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
        }
        public override object OnRestfulQueryResponse(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulQueryResponse<Sample> rtns)
        {
            foreach (var item in rtns.Results)
            {
                item.CurrentLocation = location;
            }
            return base.OnRestfulQueryResponse(req, res, dbConnection, rtns);
        }
    }

}
